@@ -2,28 +2,30 @@
 #include <math.h>
 #include <gtk/gtk.h>
 #include "gpkplotting.h"
-#include "globalflags.h"
 #include "util.h"
-#include "encoder.h"
-#include "psymodel.h"
 #include "gtkanal.h"
 #include "version.h"
 #include "lame.h"
+#include "tables.h"
+#include "quantize-pvt.h"
+#include <assert.h>
 
+int gtkflag;
 
-extern char inPath[MAX_NAME_SIZE];
 extern int makeframe(void);
 
 /* global variables for the state of the system */
 static gint idle_keepgoing;        /* processing of frames is ON */
 static gint idle_count_max;   /* number of frames to process before plotting */
 static gint idle_count;       /* pause & plot when idle_count=idel_count_max */
-static gint idle_finish=0;      /* process all frames, finish mp3 file and exit */
+static gint idle_end=0;      /* process all frames, stop at last frame  */
 static gint idle_back = 0;     /* set when we are displaying the old data */
 static int mp3done = 0;         /* last frame has been read */
 static GtkWidget *frameprogress; /* progress bar */ 
 static GtkWidget *framecounter;  /* progress counter */ 
 
+static int subblock_draw[3] = { 1, 1, 1 };
+
 /* main window */
 GtkWidget *window;
 /* Backing pixmap for drawing areas */
@@ -31,6 +33,7 @@ GtkWidget *pcmbox;       /* PCM data plotted here */
 GtkWidget *winbox;       /* mpg123 synthesis data plotted here */
 GtkWidget *enerbox[2];   /* spectrum, gr=0,1 plotted here */
 GtkWidget *mdctbox[2];   /* mdct coefficients gr=0,1 plotted here */
+GtkWidget *sfbbox[2];    /* scalefactors gr=0,1 plotted here */
 GtkWidget *headerbox;    /* mpg123 header info shown here */
 
 plotting_data *pinfo,*pplot;
@@ -51,43 +54,75 @@ struct gtkinfostruct {
   int totshort;           /* total granules with short blocks */
   int totmix;             /* total granules with mixed blocks */
   int pupdate;            /* plot while processing, or only when needed */
+  int sfblines;           /* plot scalefactor bands in MDCT plot */
+  int totalframes;
 } gtkinfo;
 
 
+static lame_global_flags *gfp;
 
 /**********************************************************************
  * read one frame and encode it 
  **********************************************************************/
-int gtkmakeframe()
+int gtkmakeframe(void)
 {
-  int iread;
+  int iread = 0;
+  static int init=0;
   static int mpglag;
   static short int Buffer[2][1152];
-  int ch,j,adj;
-  int mp3count,mp3out;
+  int ch,j;
+  int mp3count = 0;
+  int mp3out = 0;
   short mpg123pcm[2][1152];
   char mp3buffer[LAME_MAXMP3BUFFER];
   
-  iread=lame_readframe(Buffer);
-  /* even if iread=0, get_audio hit EOF and returned Buffer=all 0's.  call
-   * makeframe() one last time to flush all buffers */
-  if (input_format == sf_mp3) frameNum++;
-  else {
-    if (frameNum==0) {
-      mpglag=1;
-      lame_decode_init();
+
+#ifndef HAVEMPGLIB
+  fprintf(stderr,"Error: GTK frame analyzer requires MPGLIB\n");
+  exit(1);
+#else
+  /* even if iread=0, get_audio hit EOF and returned Buffer=all 0's.  encode
+   * and decode to flush any previous buffers from the decoder */
+
+  pinfo->frameNum = gfp->frameNum;
+  pinfo->sampfreq=gfp->out_samplerate;
+  pinfo->framesize=576*gfp->mode_gr;
+  pinfo->stereo = gfp->stereo;
+
+  if (gfp->input_format == sf_mp3) {
+    iread=lame_readframe(gfp,Buffer);
+    gfp->frameNum++;
+  }else {
+    while (gfp->frameNum == pinfo->frameNum) {
+      if (gfp->frameNum==0 && !init) {
+	mpglag=1;
+	lame_decode_init();
+      }
+      if (gfp->frameNum==1) init=0; /* reset for next time frameNum==0 */
+      iread=lame_readframe(gfp,Buffer);
+      
+      
+      mp3count=lame_encode(gfp,Buffer,mp3buffer,sizeof(mp3buffer)); /* encode frame */
+      assert( !(mp3count > 0 && gfp->frameNum == pinfo->frameNum));
+      /* not possible to produce mp3 data without encoding at least 
+       * one frame of data which would increment gfp->frameNum */
     }
-    mp3count=lame_encode(Buffer,mp3buffer); /* encode frame */
-    mp3out=lame_decode(mp3buffer,mp3count,mpg123pcm); /* re-synthesis to pcm */
-    if (mp3out>0) {
-      /* decoded output is for frame pinfo->frameNum123 */
+    mp3out=lame_decode(mp3buffer,mp3count,mpg123pcm[0],mpg123pcm[1]); /* re-synthesis to pcm */
+    /* mp3out = 0:  need more data to decode */
+    /* mp3out = -1:  error.  Lets assume 0 pcm output */
+    /* mp3out = number of samples output */
+    if (mp3out>0) assert(mp3out==pinfo->framesize);
+    if (mp3out!=0) {
+      /* decoded output is for frame pinfo->frameNum123 
+       * add a delay of framesize-DECDELAY, which will make the total delay
+       * exactly one frame */
       pinfo->frameNum123=pinfo->frameNum-mpglag;
-      adj = 1152-pinfo->framesize;
       for ( ch = 0; ch < pinfo->stereo; ch++ ) {
-	for ( j = 0; j < DECDELAY-adj; j++ )
-	  pinfo->pcmdata2[ch][j] = pinfo->pcmdata2[ch][j+1152-adj];
-	for ( j = 0; j < 1152-adj; j++ ) {
-	  pinfo->pcmdata2[ch][j+DECDELAY-adj] = mpg123pcm[ch][j];
+	for ( j = 0; j < pinfo->framesize-DECDELAY; j++ )
+	  pinfo->pcmdata2[ch][j] = pinfo->pcmdata2[ch][j+pinfo->framesize];
+	for ( j = 0; j < pinfo->framesize; j++ ) {
+	  pinfo->pcmdata2[ch][j+pinfo->framesize-DECDELAY] = 
+	    (mp3out==-1) ? 0 : mpg123pcm[ch][j];
 	}
       }
     }else{
@@ -99,6 +134,7 @@ int gtkmakeframe()
       pinfo->frameNum123=-1;  /* no frame output */
     }
   }
+#endif
   return iread;
 }
 
@@ -110,11 +146,13 @@ void plot_frame(void)
   gdouble xmx,xmn,ymx,ymn;
   double *data,*data2,*data3;
   char title2[80];
+  char label[80],label2[80];
   char *title;
   plotting_data *pplot1;
-  plotting_data *pplot2;
+  plotting_data *pplot2 = NULL;
 
   double en,samp;
+  int sampindex,version=0;
   static int firstcall=1;
   static GdkColor *barcolor,*color,*grcolor[2];
   static GdkColor yellow,gray,cyan,magenta,orange,pink,red,green,blue,black,oncolor,offcolor;
@@ -170,6 +208,9 @@ void plot_frame(void)
   if (pplot1->sampfreq)
     samp=pplot1->sampfreq;
   else samp=1;
+  sampindex = SmpFrqIndex((long)samp, &version);
+
+  ch = gtkinfo.chflag;
   
   headbits = 32 + ((pplot1->stereo==2) ? 256 : 136);
   gtkinfo.approxbits = (pplot1->bitrate*1000*1152.0/samp) - headbits;
@@ -181,16 +222,27 @@ void plot_frame(void)
   gtk_text_set_point(GTK_TEXT(headerbox),0);
   gtk_text_insert(GTK_TEXT(headerbox),NULL,&oncolor,NULL,title2, -1);
   title = " mono ";
-  if (2==pplot1->stereo) title = pplot1->js ? " jstereo " : " stereo ";
+  if (2==pplot1->stereo) title = pplot1->js ? " js " : " s ";
   gtk_text_insert (GTK_TEXT(headerbox), NULL, &oncolor, NULL,title, -1);
   color = pplot1->ms_stereo ? &oncolor : &offcolor ; 
-  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"ms_stereo ", -1);
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"ms ", -1);
   color = pplot1->i_stereo ? &oncolor : &offcolor ; 
-  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"i_stereo ", -1);
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"is ", -1);
+
+  color = pplot1->crc ? &oncolor : &offcolor ; 
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"crc ", -1);
+  color = pplot1->padding ? &oncolor : &offcolor ; 
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"pad ", -1);
+
   color = pplot1->emph ? &oncolor : &offcolor ; 
-  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"emph ", -1);
-  color = pplot1->scfsi ? &oncolor : &offcolor ; 
-  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"scfsi ", -1);
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,"em ", -1);
+
+  sprintf(title2,"c1=%i,%i ",pplot1->big_values[0][ch],pplot1->big_values[1][ch]);
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, &black, NULL,title2, -1);
+
+  color = pplot1->scfsi[ch] ? &oncolor : &offcolor ; 
+  sprintf(title2,"scfsi=%i            ",pplot1->scfsi[ch]);
+  gtk_text_insert (GTK_TEXT(headerbox), NULL, color, NULL,title2, -1);
   if (gtkinfo.filetype) 
     sprintf(title2," mdb=%i %i/NA",pplot1->maindata,pplot1->totbits);
   else
@@ -200,7 +252,7 @@ void plot_frame(void)
   gtk_text_thaw (GTK_TEXT(headerbox));
 
 
-  ch = gtkinfo.chflag;
+
   /*******************************************************************
    * block type
    *******************************************************************/
@@ -223,6 +275,12 @@ void plot_frame(void)
   else 
     title=ch ? "Right Channel" : "Left Channel";
 
+  sprintf(title2,"%s  mask_ratio=%3.2f  %3.2f  ener_ratio=%3.2f  %3.2f",
+	  title,
+	  pplot->ms_ratio[0],pplot->ms_ratio[1],
+	  pplot->ms_ener_ratio[0],pplot->ms_ener_ratio[1]);
+
+
   ymn = -32767 ; 
   ymx =  32767;
   xmn = 0;
@@ -234,7 +292,7 @@ void plot_frame(void)
    */
 
   /* draw the title */
-  gpk_graph_draw(pcmbox,0,xcord,ycord,xmn,ymn,xmx,ymx,1,title,
+  gpk_graph_draw(pcmbox,0,xcord,ycord,xmn,ymn,xmx,ymx,1,title2,
 		 &black);
 
 
@@ -275,12 +333,12 @@ void plot_frame(void)
   if (!gtkinfo.filetype) {
     n = 224;    /* number of points on end of blue part */
     /* data left of frame */
-    gpk_graph_draw(pcmbox,n+1,xcord,ycord,xmn,ymn,xmx,ymx,0,title,&black);
+    gpk_graph_draw(pcmbox,n+1,xcord,ycord,xmn,ymn,xmx,ymx,0,title2,&black);
     /* data right of frame */
     gpk_graph_draw(pcmbox,n+1,&xcord[1152+n-1],&ycord[1152+n-1],
-		   xmn,ymn,xmx,ymx,0,title,&black);
+		   xmn,ymn,xmx,ymx,0,title2,&black);
     /* the actual frame */
-    gpk_graph_draw(pcmbox,1152,&xcord[n],&ycord[n],xmn,ymn,xmx,ymx,0,title,&black);
+    gpk_graph_draw(pcmbox,1152,&xcord[n],&ycord[n],xmn,ymn,xmx,ymx,0,title2,&black);
   }
 
 
@@ -288,10 +346,12 @@ void plot_frame(void)
   /* draw the PCM re-synthesis data */
   /*******************************************************************/
   n = 1152;
-
-  sprintf(title2,"Re-synthesis  ms_ratio=%3.2f  %3.2f",
-	  pplot->ms_ratio[0],pplot->ms_ratio[1]);
-  title=title2;
+  /*
+  sprintf(title2,"Re-synthesis  mask_ratio=%3.2f  %3.2f  ener_ratio=%3.2f  %3.2f",
+	  pplot->ms_ratio[0],pplot->ms_ratio[1],
+	  pplot->ms_ener_ratio[0],pplot->ms_ener_ratio[1]);
+  */
+  title="Re-synthesis";
 
 
   ymn = -32767 ; 
@@ -352,7 +412,6 @@ void plot_frame(void)
   for (gr = 0 ; gr < mode_gr ; gr ++) {
     int bits;
     char *blockname="";
-    char label[80],label2[80];
     switch (blocktype[gr][ch]) {
     case 0: blockname = "normal"; 	break;
     case 1:  	blockname = "start";	break;
@@ -361,18 +420,6 @@ void plot_frame(void)
     }
     strcpy(label,blockname);
     if (pplot1->mixed[gr][ch]) strcat(label,"(mixed)");
-    if (blocktype[gr][ch]==2) {
-      sprintf(label2,
-	      "%i %i%i%i",
-	      pplot1->scalefac_scale[gr][ch],
-              pplot1->sub_gain[gr][ch][0],
-	      pplot1->sub_gain[gr][ch][1],
-	      pplot1->sub_gain[gr][ch][2]);
-      strcat(label,label2);
-    }else{
-      sprintf(label2,"%i",pplot1->scalefac_scale[gr][ch]);
-      strcat(label,label2);
-    }
 
     
     
@@ -387,17 +434,53 @@ void plot_frame(void)
     }
     
 
+    xmn = 0;
+    xmx = n-1;
+    ymn=0;
+    ymx=11;
+
+    /* draw title, erase old plot */
+    if (gtkinfo.flag123) bits=pplot1->mainbits[gr][ch];
+    else bits=pplot->LAMEmainbits[gr][ch];
+    sprintf(title2,"MDCT%1i(%s) bits=%i q=%i ",gr,label,bits,
+	      pplot1->qss[gr][ch]);
+    gpk_bargraph_draw(mdctbox[gr],0,xcord,ycord,
+		      xmn,ymn,xmx,ymx,1,title2,0,barcolor);
+
+    /* draw some hash marks showing scalefactor bands */
+    if (gtkinfo.sfblines) {
+      int fac,nsfb, *scalefac;
+      if (blocktype[gr][ch]==SHORT_TYPE) {
+	nsfb=SBMAX_s;
+	fac=3;
+	scalefac = scalefac_band.s;
+      }else{
+	nsfb=SBMAX_l;
+	fac=1;
+	scalefac = scalefac_band.l;
+      }
+      for (i=nsfb-7 ; i<nsfb; i++) {
+	ycord[0] = .8*ymx;  ycord[1] = ymn;
+	xcord[0] = fac*scalefac[i];
+	xcord[1] = xcord[0];
+	gpk_rectangle_draw(mdctbox[gr],xcord,ycord,xmn,ymn,xmx,ymx,&yellow);
+      }
+    }   
+
+
 
     ymn=9e20;
     ymx=-9e20;
     for (i=0; i<n; i++) {
       double coeff;
-      xcord[i] = i+1;
+      xcord[i] = i;
       if (gtkinfo.msflag){
 	coeff = ch ?  .5*(data[i]-data2[i]) : .5*(data[i]+data2[i]) ;
       }else{
 	coeff = ch ? data2[i] : data[i];
       }
+      if (blocktype[gr][ch]==SHORT_TYPE && !subblock_draw[i % 3])
+        coeff = 0;
       ycord[i]=coeff*coeff*1e10;
       ycord[i] = log10( MAX( ycord[i],(double) 1)); 
       ymx=(ycord[i] > ymx) ? ycord[i] : ymx;
@@ -412,14 +495,14 @@ void plot_frame(void)
     
     
     sprintf(title2,"MDCT%1i(%s) bits=%i q=%i ",gr,label,bits,
-	      pplot1->qss[gr][ch]-210);
-    
-    xmn = 1;
-    xmx = n;
+	      pplot1->qss[gr][ch]);
+
+    xmn = 0;
+    xmx = n-1;
     ymn=0;
     ymx=11;
     gpk_bargraph_draw(mdctbox[gr],n,xcord,ycord,
-		      xmn,ymn,xmx,ymx,1,title2,0,barcolor);
+		      xmn,ymn,xmx,ymx,0,title2,0,barcolor);
   }
   
 
@@ -428,18 +511,8 @@ void plot_frame(void)
   /*******************************************************************
    * draw the psy model energy spectrum (k space) 
    * l3psy.c computes pe, en, thm for THIS granule.  
-   * To encode, ISO uses (bug): en,thm from last granule.  
-   *                            pe from this granule
-   * gpsycho model corrects this
    *******************************************************************/
-  if (gtkinfo.msflag || gtkinfo.filetype) {
-    /* erase everything */
-    gpk_bargraph_draw(enerbox[0],0,xcord,ycord,
-		      xmn,ymn,xmx,ymx,1," ",0,barcolor);
-    gpk_bargraph_draw(enerbox[1],0,xcord,ycord,
-		      xmn,ymn,xmx,ymx,1," ",0,barcolor);
-  }
-  else if (gtkinfo.kbflag){
+ if (gtkinfo.kbflag){
     for (gr = 0 ; gr < mode_gr ; gr ++) {
       n = HBLKSIZE; /* only show half the spectrum */
 
@@ -449,7 +522,10 @@ void plot_frame(void)
       ymx=-9e20;
       for (i=0; i<n; i++) {
 	xcord[i] = i+1;
-	ycord[i] = log10( MAX( data[i],(double) 1));
+        if (blocktype[gr][ch]==SHORT_TYPE && !subblock_draw[i % 3])
+          ycord[i] = 0;
+        else
+	  ycord[i] = log10( MAX( data[i],(double) 1));
 	ymx=(ycord[i] > ymx) ? ycord[i] : ymx;
 	ymn=(ycord[i] < ymn) ? ycord[i] : ymn;
       }
@@ -470,8 +546,6 @@ void plot_frame(void)
   }else{
     /*******************************************************************
      * draw the psy model energy spectrum (scalefactor bands)
-     * ISO model (because of bugs) uses:  
-     *        pe,en,thm,energy delayed 1 granule
      *******************************************************************/
     for (gr = 0 ; gr < mode_gr ; gr ++) {
 
@@ -490,7 +564,10 @@ void plot_frame(void)
       ymx=-9e20;
       for (i=0; i<n; i++) {
 	xcord[i] = i+1;
-	ycord[i] = log10( MAX( data[i],(double) 1));
+        if (blocktype[gr][ch]==SHORT_TYPE && !subblock_draw[i % 3])
+          ycord[i] = 0;
+        else
+	  ycord[i] = log10( MAX( data[i],(double) 1));
 	ymx=(ycord[i] > ymx) ? ycord[i] : ymx;
 	ymn=(ycord[i] < ymn) ? ycord[i] : ymn;
       }
@@ -516,7 +593,10 @@ void plot_frame(void)
 
       for (i=0; i<n; i++) {
 	xcord[i] = i+1;
-	ycord[i] = log10( MAX( data3[i], (double) 1));
+        if (blocktype[gr][ch]==SHORT_TYPE && !subblock_draw[i % 3])
+          ycord[i] = 0;
+        else
+	  ycord[i] = log10( MAX( data3[i], (double) 1));
 	ymx=(ycord[i] > ymx) ? ycord[i] : ymx;
 	ymn=(ycord[i] < ymn) ? ycord[i] : ymn;
       }
@@ -526,7 +606,10 @@ void plot_frame(void)
       
       for (i=0; i<n; i++) {
 	xcord[i] = i+1 + (.25*n)/SBMAX_l;
-	ycord[i] = log10( MAX( data2[i], (double) 1));
+        if (blocktype[gr][ch]==SHORT_TYPE && !subblock_draw[i % 3])
+          ycord[i] = 0;
+        else
+	  ycord[i] = log10( MAX( data2[i], (double) 1));
 	ymx=(ycord[i] > ymx) ? ycord[i] : ymx;
 	ymn=(ycord[i] < ymn) ? ycord[i] : ymn;
       }
@@ -534,6 +617,65 @@ void plot_frame(void)
 			xmn,ymn,xmx,ymx,0,title2,3,grcolor[gr]);
     }
   }
+
+  /*******************************************************************
+   * draw scalefactors 
+   *******************************************************************/
+  for (gr = 0 ; gr < mode_gr ; gr ++) {
+      double ggain;
+      if (blocktype[gr][ch]==2) {
+	n = 3*SBMAX_s; 
+	if (gtkinfo.flag123) data = pplot1->sfb_s[gr][ch];
+	else data = pplot->LAMEsfb_s[gr][ch];
+      } else {
+	n = SBMAX_l; 
+	if (gtkinfo.flag123) data = pplot1->sfb[gr][ch];
+	else data = pplot->LAMEsfb[gr][ch];
+      }
+
+      ymn=-1;
+      ymx=10;
+      for (i=0; i<n; i++) {
+	xcord[i] = i+1;
+        if (blocktype[gr][ch]==SHORT_TYPE && !subblock_draw[i % 3])
+          ycord[i] = 0;
+        else
+	  ycord[i] = -data[i];
+	ymx=(ycord[i] > ymx) ? ycord[i] : ymx;
+	ymn=(ycord[i] < ymn) ? ycord[i] : ymn;
+      }
+
+      if (blocktype[gr][ch]==2) {
+	sprintf(label2,
+		"SFB scale=%i %i%i%i",
+		pplot1->scalefac_scale[gr][ch],
+		pplot1->sub_gain[gr][ch][0],
+		pplot1->sub_gain[gr][ch][1],
+		pplot1->sub_gain[gr][ch][2]);
+      }else{
+	sprintf(label2,"SFB scale=%i",pplot1->scalefac_scale[gr][ch]);
+      }
+      
+      if (gtkinfo.flag123) ggain = -(pplot1->qss[gr][ch]-210)/4.0;
+      else ggain = -(pplot->LAMEqss[gr][ch]-210)/4.0;
+
+      sprintf(title2," gain=%4.1f",ggain);
+      strcat(label2,title2);
+      
+      xmn = 1;
+      xmx = n+1;
+      gpk_bargraph_draw(sfbbox[gr],n,xcord,ycord,
+			xmn,ymn,xmx,ymx,1,label2,0,grcolor[gr]);
+
+      ycord[0] = ycord[1] = 0;
+      xcord[0] = 1;
+      xcord[1] = n+1;
+      gpk_rectangle_draw(sfbbox[gr],xcord,ycord,xmn,ymn,xmx,ymx,&yellow);
+
+      
+    }
+
+
 }
 
 
@@ -541,8 +683,11 @@ void plot_frame(void)
 static void update_progress(void)
 {    
   char label[80];
+  int tf=gfp->totalframes;
+  if (gtkinfo.totalframes>0) tf=gtkinfo.totalframes;
+
   sprintf(label,"Frame:%4i/%4i  %6.2fs",
-	 pplot->frameNum,(int)totalframes-1, pplot->frametime);
+	 pplot->frameNum,(int)tf-1, pplot->frametime);
   gtk_progress_set_value (GTK_PROGRESS (frameprogress), (gdouble) pplot->frameNum);
   gtk_label_set_text(GTK_LABEL(framecounter),label);
 }
@@ -555,7 +700,7 @@ static void analyze(void)
       idle_count = 0;
       idle_count_max=0;
       idle_keepgoing=0;
-      idle_finish=0;
+      idle_end=0;
     }
     plot_frame();   
     update_progress(); 
@@ -567,7 +712,7 @@ static void plotclick( GtkWidget *widget, gpointer   data )
 
 
 
-static int frameadv1( GtkWidget *widget, gpointer   data )
+static int frameadv1(GtkWidget *widget, gpointer   data )
 {
   int i;
   if (idle_keepgoing ){
@@ -580,23 +725,26 @@ static int frameadv1( GtkWidget *widget, gpointer   data )
       pplot = &Pinfo[READ_AHEAD];
       if (mp3done) { 
 	/* dont try to read any more frames, and quit if "finish MP3" was selected */
-	if (idle_finish) gtk_main_quit();
+	/*	if (idle_finish) gtk_main_quit(); */
 	idle_count_max=0; 
+        idle_end=0;
       } else {
 	/* read in the next frame */
 	for (i=NUMPINFO-1 ; i>0 ; i--)
 	  memcpy(&Pinfo[i],&Pinfo[i-1],sizeof(plotting_data));
 	pinfo = &Pinfo[0];
 	pinfo->num_samples = gtkmakeframe();
+	if (pinfo->num_samples==0 && gtkinfo.totalframes==0) 
+	  /* allow an extra frame to flush decoder buffers */
+	  gtkinfo.totalframes = pinfo->frameNum +2;
 
 	if (pinfo->sampfreq) 
 	  pinfo->frametime = (pinfo->frameNum)*1152.0/pinfo->sampfreq;
 	else pinfo->frametime=0;
 
-        /* eof? */
-	if (!pinfo->num_samples) {
-	  if (idle_finish) gtk_main_quit();
-	}
+        /* eof? 
+	if (!pinfo->num_samples) if (idle_finish) gtk_main_quit();
+	*/
 
 	pinfo->totbits = 0;
 	{ int gr,ch;
@@ -616,14 +764,15 @@ static int frameadv1( GtkWidget *widget, gpointer   data )
 	gtkinfo.totms   += !(pinfo->ms_stereo==0);
 	gtkinfo.totis   += !(pinfo->i_stereo==0);
 
-	if (pplot->frameNum >= totalframes-1) mp3done=1;
+	if (gtkinfo.totalframes>0)
+	  if (pplot->frameNum >= gtkinfo.totalframes-1) mp3done=1;
       }
     }
 
     idle_count++;
     if (gtkinfo.pupdate) plot_frame();
     update_progress();
-    if ((idle_count>=idle_count_max) && (! idle_finish)) analyze();
+    if ((idle_count>=idle_count_max) && (! idle_end)) analyze();
   }
   return 1;
 }
@@ -647,7 +796,7 @@ static void frameadv( GtkWidget *widget, gpointer   data )
     if (!strcmp((char *) data,"1")) adv = 1;
     if (!strcmp((char *) data,"10")) adv = 10;
     if (!strcmp((char *) data,"100")) adv = 100;
-    if (!strcmp((char *) data,"finish")) idle_finish = 1;
+    if (!strcmp((char *) data,"finish")) idle_end = 1;
 
 
     if (idle_keepgoing) {
@@ -681,8 +830,8 @@ static void delete_event( GtkWidget *widget,
 
 static void channel_option (GtkWidget *widget, gpointer data)
 {
-  int option;
-  option = (int) data;
+  long option;
+  option = (long) data;
   switch (option) {
   case 1:
     gtkinfo.msflag=0;
@@ -704,8 +853,8 @@ static void channel_option (GtkWidget *widget, gpointer data)
 }
 static void spec_option (GtkWidget *widget, gpointer data)
 {
-  int option;
-  option = (int) data;
+  long option;
+  option = (long) data;
   switch (option) {
   case 1:
     gtkinfo.kbflag=0;
@@ -725,20 +874,53 @@ static void spec_option (GtkWidget *widget, gpointer data)
   case 6:
     gtkinfo.pupdate=0;
     break;
+  case 7:
+    gtkinfo.sfblines = !gtkinfo.sfblines;
+    break;
   }
   analyze();
 }
 
+static gint key_press_event (GtkWidget *widget, GdkEventKey *event)
+{
+  if (event->keyval == '1') {
+    subblock_draw[0] = 1;
+    subblock_draw[1] = 0;
+    subblock_draw[2] = 0;
+    analyze();
+  }
+  else if (event->keyval == '2') {
+    subblock_draw[0] = 0;
+    subblock_draw[1] = 1;
+    subblock_draw[2] = 0;
+    analyze();
+  }
+  else if (event->keyval == '3') {
+    subblock_draw[0] = 0;
+    subblock_draw[1] = 0;
+    subblock_draw[2] = 1;
+    analyze();
+  }
+  else if (event->keyval == '0') {
+    subblock_draw[0] = 1;
+    subblock_draw[1] = 1;
+    subblock_draw[2] = 1;
+    analyze();
+  }
+  /* analyze(); */  /* dont redraw entire window for every key! */
+  return 0;
+}
+
 
 
 static void text_window (GtkWidget *widget, gpointer data)
 {
-  int option;
+  long option;
   GtkWidget *hbox,*vbox,*button,*box;
   GtkWidget *textwindow,*vscrollbar;
   char text[80];
 
-  option = (int) data;
+  option = (long) data;
   
   textwindow = gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
@@ -951,6 +1133,7 @@ static GtkItemFactoryEntry menu_items[] = {
   {"/_MDCT",                         NULL,  NULL, 0, "<Branch>"},
   {"/MDCT/_Original",               NULL,  spec_option, 3, NULL},
   {"/MDCT/_Compressed",             NULL,  spec_option, 4, NULL},
+  {"/MDCT/_Toggle SFB lines",       NULL,  spec_option, 7, NULL},
 
   {"/_Stats",                         NULL,  NULL, 0, "<Branch>"},
   {"/Stats/_Show",               NULL,  text_window, 2, NULL},
@@ -995,7 +1178,7 @@ static void get_main_menu(GtkWidget *window, GtkWidget ** menubar) {
 
 
 
-int gtkcontrol(void)
+int gtkcontrol(lame_global_flags *gfp2)
 {
     /* GtkWidget is the storage type for widgets */
     GtkWidget *button;
@@ -1011,15 +1194,16 @@ int gtkcontrol(void)
     char frameinfo[80];
 
     graphx = 500;  /* minimum allowed size of pixmap */
-    graphy = 100;
+    graphy = 95;
 
+    gfp=gfp2;
 
     /* set some global defaults/variables */
-    gtkinfo.filetype = (input_format == sf_mp3);
+    gtkinfo.filetype = (gfp->input_format == sf_mp3);
     gtkinfo.msflag=0;
     gtkinfo.chflag=0;
     gtkinfo.kbflag=0;
-    gtkinfo.flag123 = (input_format == sf_mp3); /* MP3 file=use mpg123 output */
+    gtkinfo.flag123 = (gfp->input_format == sf_mp3); /* MP3 file=use mpg123 output */
     gtkinfo.pupdate=0;
     gtkinfo.avebits = 0;
     gtkinfo.maxbits = 0;
@@ -1029,19 +1213,24 @@ int gtkcontrol(void)
     gtkinfo.totis = 0;
     gtkinfo.totshort = 0;
     gtkinfo.totmix = 0;
-
-    totalframes=Max(totalframes,READ_AHEAD+1);
+    gtkinfo.sfblines= 1;
+    gtkinfo.totalframes = 0;
 
     memset((char *) Pinfo, 0, sizeof(Pinfo));
     pplot = &Pinfo[READ_AHEAD];
 
     strcpy(frameinfo,"MP3x: ");
-    strncat(frameinfo,inPath,70);
+    strncat(frameinfo,gfp->inPath,70);
 
     window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title (GTK_WINDOW (window), frameinfo);
     gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 			GTK_SIGNAL_FUNC (delete_event), NULL);
+
+    gtk_signal_connect_object (GTK_OBJECT (window), "key_press_event",
+		      GTK_SIGNAL_FUNC(key_press_event),
+		      GTK_OBJECT (window));
+
     gtk_container_set_border_width (GTK_CONTAINER (window), 0);
 
 
@@ -1052,7 +1241,7 @@ int gtkcontrol(void)
     box1 = gtk_hbox_new(FALSE, 0);
     box2 = gtk_hbox_new(FALSE, 0);
     box3 = gtk_hbox_new(FALSE, 0);
-    table = gtk_table_new (4, 2, FALSE);
+    table = gtk_table_new (5, 2, FALSE);
     tableops = GTK_FILL | GTK_EXPAND | GTK_SHRINK;
     get_main_menu(window, &menubar);
 
@@ -1086,7 +1275,7 @@ int gtkcontrol(void)
     gtk_widget_show(framecounter);
     gtk_box_pack_start(GTK_BOX (box2),framecounter, FALSE, TRUE, 0);
 
-    adj = (GtkAdjustment *) gtk_adjustment_new (0, 0,(gint) totalframes-1, 0, 0, 0);
+    adj = (GtkAdjustment *) gtk_adjustment_new (0, 0,(gint) gfp->totalframes-1, 0, 0, 0);
     frameprogress = gtk_progress_bar_new_with_adjustment (adj);
     /* Set the format of the string that can be displayed in the
      * trough of the progress bar:
@@ -1130,7 +1319,7 @@ int gtkcontrol(void)
     gtk_box_pack_start(GTK_BOX(box1), button, TRUE, TRUE, 0);
     gtk_widget_show(button);
 
-    button = gtk_button_new_with_label ("finish mp3");
+    button = gtk_button_new_with_label ("last frame");
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (frameadv), (gpointer) "finish");
     gtk_box_pack_start(GTK_BOX(box1), button, TRUE, TRUE, 0);
@@ -1171,6 +1360,14 @@ int gtkcontrol(void)
     gtk_table_attach (GTK_TABLE(table),enerbox[1],1,2,3,4,tableops,tableops,2,2);
     gtk_widget_show (enerbox[1]);
 
+    sfbbox[0] = gpk_plot_new(graphy,graphy);
+    gtk_table_attach(GTK_TABLE(table),sfbbox[0],0,1,4,5,tableops,tableops,2,2);
+    gtk_widget_show (sfbbox[0]);
+
+    sfbbox[1] = gpk_plot_new(graphy,graphy);
+    gtk_table_attach (GTK_TABLE(table),sfbbox[1],1,2,4,5,tableops,tableops,2,2);
+    gtk_widget_show (sfbbox[1]);
+
 
 
 
