@@ -1,3 +1,4 @@
+#ifdef HAVEMPGLIB
 /* 
  * Mpeg Layer-3 audio decoder 
  * --------------------------
@@ -9,13 +10,10 @@
 #include "mpg123.h"
 #include "mpglib.h"
 #include "huffman.h"
-#ifdef OS_AMIGAOS
-#include "/gtkanal.h"
-#else
+#ifdef HAVEGTK
 #include "../gtkanal.h"
 #endif
 
-
 extern struct mpstr *gmp;
 
 #define MPEG1
@@ -65,8 +63,8 @@ struct bandInfoStruct bandInfo[9] = {
    {6,6,6,6,6,6,8,10,12,14,16,20,24,28,32,38,46,52,60,68,58,54 } ,
    {0,4*3,8*3,12*3,18*3,24*3,32*3,42*3,56*3,74*3,100*3,132*3,174*3,192*3} ,
    {4,4,4,6,6,8,10,14,18,26,32,42,18 } } ,
-
- { {0,6,12,18,24,30,36,44,54,66,80,96,114,136,162,194,232,278,330,394,464,540,576},
+                                             /* docs: 332. mpg123: 330 */
+ { {0,6,12,18,24,30,36,44,54,66,80,96,114,136,162,194,232,278,332,394,464,540,576},
    {6,6,6,6,6,6,8,10,12,14,16,18,22,26,32,38,46,52,64,70,76,36 } ,
    {0,4*3,8*3,12*3,18*3,26*3,36*3,48*3,62*3,80*3,104*3,136*3,180*3,192*3} ,
    {4,4,4,6,8,10,12,14,18,24,32,44,12 } } ,
@@ -348,8 +346,10 @@ static void III_get_side_info_1(struct III_sideinfo *si,int stereo,
 	 unsigned int qss = getbits_fast(8);
 	 gr_info->pow2gain = gainpow2+256 - qss + powdiff;
 #ifdef HAVEGTK
-	 if (gtkflag) 
+	 if (gtkflag) {
 	   pinfo->qss[gr][ch]=qss;
+	   pinfo->big_values[gr][ch]=gr_info->big_values;
+	 }
 #endif
        }
        if(ms_stereo)
@@ -430,6 +430,7 @@ static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
    for (ch=0; ch<stereo; ch++) 
    {
        register struct gr_info_s *gr_info = &(si->ch[ch].gr[0]);
+       unsigned int qss;
 
        gr_info->part2_3_length = getbits(12);
        gr_info->big_values = getbits_fast(9);
@@ -437,7 +438,15 @@ static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
          fprintf(stderr,"big_values too large!\n");
          gr_info->big_values = 288;
        }
-       gr_info->pow2gain = gainpow2+256 - getbits_fast(8) + powdiff;
+       qss=getbits_fast(8);
+       gr_info->pow2gain = gainpow2+256 - qss + powdiff;
+#ifdef HAVEGTK
+       if (gtkflag) {
+	   pinfo->qss[0][ch]=qss;
+	   pinfo->big_values[0][ch]=gr_info->big_values;
+       }
+#endif
+
        if(ms_stereo)
          gr_info->pow2gain += 2;
        gr_info->scalefac_compress = getbits(9);
@@ -455,7 +464,12 @@ static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
           */
          gr_info->table_select[2] = 0;
          for(i=0;i<3;i++) {
-           gr_info->full_gain[i] = gr_info->pow2gain + (getbits_fast(3)<<3);
+	   unsigned int sbg = (getbits_fast(3)<<3);
+           gr_info->full_gain[i] = gr_info->pow2gain + sbg;
+#ifdef HAVEGTK
+	   if (gtkflag)
+	     pinfo->sub_gain[0][ch][i]=sbg/8;
+#endif
 	 }
 
          if(gr_info->block_type == 0) {
@@ -546,8 +560,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_info)
           numbits += num0 * 6;
         }
         else {
-          *scf++ = 0; *scf++ = 0; *scf++ = 0;  /* set to ZERO necessary? */
-          *scf++ = 0; *scf++ = 0; *scf++ = 0;
+          scf += 6;
         }
 
         if(!(scfsi & 0x4)) {
@@ -556,8 +569,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_info)
           numbits += num0 * 5;
         }
         else {
-          *scf++ = 0; *scf++ = 0; *scf++ = 0;  /* set to ZERO necessary? */
-          *scf++ = 0; *scf++ = 0;
+          scf += 5;
         }
 
         if(!(scfsi & 0x2)) {
@@ -566,8 +578,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_info)
           numbits += num1 * 5;
         }
         else {
-          *scf++ = 0; *scf++ = 0; *scf++ = 0;  /* set to ZERO necessary? */
-          *scf++ = 0; *scf++ = 0;
+          scf += 5;
         }
 
         if(!(scfsi & 0x1)) {
@@ -576,8 +587,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_info)
           numbits += num1 * 5;
         }
         else {
-          *scf++ = 0; *scf++ = 0; *scf++ = 0;  /* set to ZERO necessary? */
-          *scf++ = 0; *scf++ = 0;
+          scf += 5;
         }
       }
 
@@ -1503,7 +1513,7 @@ static void III_hybrid(real fsIn[SBLIMIT][SSLIMIT],real tsOut[SSLIMIT][SBLIMIT],
 int do_layer3(struct frame *fr,unsigned char *pcm_sample,int *pcm_point)
 {
   int gr, ch, ss,clip=0;
-  int scalefacs[39]; /* max 39 for short[13][3] mode, mixed: 38, long: 22 */
+  int scalefacs[2][39]; /* max 39 for short[13][3] mode, mixed: 38, long: 22 */
   struct III_sideinfo sideinfo;
   int stereo = fr->stereo;
   int single = fr->single;
@@ -1556,31 +1566,45 @@ int do_layer3(struct frame *fr,unsigned char *pcm_sample,int *pcm_point)
       struct gr_info_s *gr_info = &(sideinfo.ch[0].gr[gr]);
       long part2bits;
       if(fr->lsf)
-        part2bits = III_get_scale_factors_2(scalefacs,gr_info,0);
+        part2bits = III_get_scale_factors_2(scalefacs[0],gr_info,0);
       else {
 #ifdef MPEG1
-        part2bits = III_get_scale_factors_1(scalefacs,gr_info);
+        part2bits = III_get_scale_factors_1(scalefacs[0],gr_info);
 #else
 	fprintf(stderr,"Not supported\n");
 #endif
       }
-      if(III_dequantize_sample(hybridIn[0], scalefacs,gr_info,sfreq,part2bits))
+#ifdef HAVEGTK
+      if (gtkflag) {
+	int i;
+	for (i=0; i<39; i++) 
+	  pinfo->sfb_s[gr][0][i]=scalefacs[0][i];
+      }
+#endif
+      if(III_dequantize_sample(hybridIn[0], scalefacs[0],gr_info,sfreq,part2bits))
         return clip;
     }
     if(stereo == 2) {
       struct gr_info_s *gr_info = &(sideinfo.ch[1].gr[gr]);
       long part2bits;
       if(fr->lsf) 
-        part2bits = III_get_scale_factors_2(scalefacs,gr_info,i_stereo);
+        part2bits = III_get_scale_factors_2(scalefacs[1],gr_info,i_stereo);
       else {
 #ifdef MPEG1
-        part2bits = III_get_scale_factors_1(scalefacs,gr_info);
+        part2bits = III_get_scale_factors_1(scalefacs[1],gr_info);
 #else
 	fprintf(stderr,"Not supported\n");
 #endif
       }
+#ifdef HAVEGTK
+      if (gtkflag) {
+	int i;
+	for (i=0; i<39; i++) 
+	  pinfo->sfb_s[gr][1][i]=scalefacs[1][i];
+      }
+#endif
 
-      if(III_dequantize_sample(hybridIn[1],scalefacs,gr_info,sfreq,part2bits))
+      if(III_dequantize_sample(hybridIn[1],scalefacs[1],gr_info,sfreq,part2bits))
           return clip;
 
       if(ms_stereo) {
@@ -1595,7 +1619,7 @@ int do_layer3(struct frame *fr,unsigned char *pcm_sample,int *pcm_point)
       }
 
       if(i_stereo)
-        III_i_stereo(hybridIn,scalefacs,gr_info,sfreq,ms_stereo,fr->lsf);
+        III_i_stereo(hybridIn,scalefacs[1],gr_info,sfreq,ms_stereo,fr->lsf);
 
       if(ms_stereo || i_stereo || (single == 3) ) {
         if(gr_info->maxb > sideinfo.ch[0].gr[gr].maxb) 
@@ -1627,24 +1651,56 @@ int do_layer3(struct frame *fr,unsigned char *pcm_sample,int *pcm_point)
 #ifdef HAVEGTK
     if (gtkflag) {
     extern int tabsel_123[2][3][16];
-    int sb;
+    extern int pretab[21];
+    int i,j,sb;
+    float ifqstep;
+
+    for (ch=0;ch<stereo1;ch++) {
+      struct gr_info_s *gr_info = &(sideinfo.ch[ch].gr[gr]);
+      ifqstep = ( pinfo->scalefac_scale[gr][ch] == 0 ) ? .5 : 1.0;
+      if (2==gr_info->block_type) {
+	for (i=0; i<3; i++) {
+	  for (sb=0; sb<12; sb++) {
+	    j = 3*sb+i;
+	    /*
+           is_p = scalefac[sfb*3+lwin-gr_info->mixed_block_flag]; 
+	    */
+	    /* scalefac was copied into pinfo->sfb_s[] above */
+	    pinfo->sfb_s[gr][ch][j] = -ifqstep*pinfo->sfb_s[gr][ch][j-gr_info->mixed_block_flag];
+	    pinfo->sfb_s[gr][ch][j] -= 2*(pinfo->sub_gain[gr][ch][i]);
+	  }
+	  pinfo->sfb_s[gr][ch][3*sb+i] = - 2*(pinfo->sub_gain[gr][ch][i]);
+	}
+      }else{
+	for (sb=0; sb<21; sb++) {
+	  /* scalefac was copied into pinfo->sfb[] above */
+	  pinfo->sfb[gr][ch][sb] = pinfo->sfb_s[gr][ch][sb];
+	  if (gr_info->preflag) pinfo->sfb[gr][ch][sb] += pretab[sb];
+	  pinfo->sfb[gr][ch][sb] *= -ifqstep;
+	}
+      }
+    }
+
+
     
     pinfo->bitrate = 
       tabsel_123[fr->lsf][fr->lay-1][fr->bitrate_index];
     pinfo->sampfreq = freqs[sfreq];
     pinfo->emph = fr->emphasis;
+    pinfo->crc = fr->error_protection;
+    pinfo->padding = fr->padding;
     pinfo->stereo = fr->stereo;
     pinfo->js =   (fr->mode == MPG_MD_JOINT_STEREO);
     pinfo->ms_stereo = ms_stereo;
     pinfo->i_stereo = i_stereo;
     pinfo->maindata = sideinfo.main_data_begin;
-    
+
     for(ch=0;ch<stereo1;ch++) {
       struct gr_info_s *gr_info = &(sideinfo.ch[ch].gr[gr]);
       pinfo->mixed[gr][ch] = gr_info->mixed_block_flag;
       pinfo->mpg123blocktype[gr][ch]=gr_info->block_type;
       pinfo->mainbits[gr][ch] = gr_info->part2_3_length;
-      pinfo->scfsi = gr_info->scfsi;
+      if (gr==1) pinfo->scfsi[ch] = gr_info->scfsi;
     }
     for(ch=0;ch<stereo1;ch++) { 
       int j=0;
@@ -1679,3 +1735,4 @@ int do_layer3(struct frame *fr,unsigned char *pcm_sample,int *pcm_point)
 
 
 
+#endif
